/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.    
 */

package org.apache.tuscany.sca.itest.sdodatabinding;

import org.apache.tuscany.sca.itest.databinding.types.PersonType;
import org.apache.tuscany.interop.AttributeComplexType;
import org.apache.tuscany.interop.AttributeReferenceComplexType;
import org.apache.tuscany.interop.SimpleTypeWithAbstractComplexType;
import org.apache.tuscany.interop.SimpleTypeWithNameComplexType;
import org.apache.tuscany.interop.ComplexTypeWithContentType;

import org.osoa.sca.annotations.Service;

/**
 * The automatically generated service implementation that reflects data objects
 * that are sent to it
 *
 * @version $Rev: 511417 $ $Date: 2007-02-25 03:06:42 +0000 (Sun, 25 Feb 2007) $
 */
@Service(GreeterService.class)
public class GreeterServiceImpl implements GreeterService {

    /**
     * Take the provided data object, change it slightly and return it
     * 
     * @param param the data object to be changed and returned
     * @return the change data object 
     */
    public PersonType greetPersonType(PersonType param) {
        
        param.setGreeting("Hello");
      
        return param;
    }

    /**
     * Take the provided data object, change it slightly and return it
     * 
     * @param param the data object to be changed and returned
     * @return the change data object 
     */
    public AttributeComplexType greetAttributeComplexType(AttributeComplexType param) {
        
        param.getAttributeElement().setAttribute("SomeChangedText");;
      
        return param;
    }

    /**
     * Take the provided data object, change it slightly and return it
     * 
     * @param param the data object to be changed and returned
     * @return the change data object 
     */
    public AttributeReferenceComplexType greetAttributeReferenceComplexType(AttributeReferenceComplexType param) {
        
        param.setReferencedAttribute("SomeChangedText");;
      
        return param;
    }

    /**
     * Take the provided data object, change it slightly and return it
     * 
     * @param param the data object to be changed and returned
     * @return the change data object 
     */
    public SimpleTypeWithAbstractComplexType greetSimpleTypeWithAbstractComplexType(SimpleTypeWithAbstractComplexType param) {
        
        param.setSimpleTypeWithAbstractExtensionElement("SomeChangedText");;
      
        return param;
    }

    /**
     * Take the provided data object, change it slightly and return it
     * 
     * @param param the data object to be changed and returned
     * @return the change data object 
     */
    public SimpleTypeWithNameComplexType greetSimpleTypeWithNameComplexType(SimpleTypeWithNameComplexType param) {
        
        param.setSimpleTypeWithNameElement("SomeChangedText");;
      
        return param;
    }

    /**
     * Take the provided data object, change it slightly and return it
     * 
     * @param param the data object to be changed and returned
     * @return the change data object 
     */
    public ComplexTypeWithContentType greetComplexTypeWithContentType(ComplexTypeWithContentType param) {
        
        param.setSimpleTypeWithName("SomeChangedText");;
      
        return param;
    }
}
