/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.    
 */
package org.apache.tuscany.sca.binding.ws.axis2;

import java.util.List;

import org.apache.tuscany.sca.binding.ws.WebServiceBinding;
import org.apache.tuscany.sca.contribution.ModelFactoryExtensionPoint;
import org.apache.tuscany.sca.core.ExtensionPointRegistry;
import org.apache.tuscany.sca.core.UtilityExtensionPoint;
import org.apache.tuscany.sca.databinding.DataBindingExtensionPoint;
import org.apache.tuscany.sca.host.http.ServletHost;
import org.apache.tuscany.sca.host.http.ServletHostExtensionPoint;
import org.apache.tuscany.sca.policy.util.PolicyHandlerDefinitionsLoader;
import org.apache.tuscany.sca.policy.util.PolicyHandlerTuple;
import org.apache.tuscany.sca.provider.BindingProviderFactory;
import org.apache.tuscany.sca.provider.ReferenceBindingProvider;
import org.apache.tuscany.sca.provider.ServiceBindingProvider;
import org.apache.tuscany.sca.runtime.RuntimeComponent;
import org.apache.tuscany.sca.runtime.RuntimeComponentReference;
import org.apache.tuscany.sca.runtime.RuntimeComponentService;
import org.apache.tuscany.sca.work.WorkScheduler;

/**
 * Axis2BindingProviderFactory
 *
 * @version $Rev: 795364 $ $Date: 2009-07-18 14:32:22 +0100 (Sat, 18 Jul 2009) $
 */

public class Axis2BindingProviderFactory implements BindingProviderFactory<WebServiceBinding> {

    private ModelFactoryExtensionPoint modelFactories;
    private List<ServletHost> hosts;
    private List<PolicyHandlerTuple> policyHandlerClassnames = null;
    private DataBindingExtensionPoint dataBindings;
    private WorkScheduler workScheduler;

    public Axis2BindingProviderFactory(ExtensionPointRegistry extensionPoints) {
        ServletHostExtensionPoint servletHosts = extensionPoints.getExtensionPoint(ServletHostExtensionPoint.class);
        hosts = servletHosts.getServletHosts();

        modelFactories = extensionPoints.getExtensionPoint(ModelFactoryExtensionPoint.class);
        policyHandlerClassnames = PolicyHandlerDefinitionsLoader.loadPolicyHandlerClassnames();
        dataBindings = extensionPoints.getExtensionPoint(DataBindingExtensionPoint.class);
        UtilityExtensionPoint utilities = extensionPoints.getExtensionPoint(UtilityExtensionPoint.class);
        workScheduler = utilities.getUtility(WorkScheduler.class);
    }

    public ReferenceBindingProvider createReferenceBindingProvider(RuntimeComponent component,
                                                                   RuntimeComponentReference reference,
                                                                   WebServiceBinding binding) {
        return new Axis2ReferenceBindingProvider(component, reference, binding,
                                                 modelFactories, policyHandlerClassnames, dataBindings);
    }

    public ServiceBindingProvider createServiceBindingProvider(RuntimeComponent component,
                                                               RuntimeComponentService service,
                                                               WebServiceBinding binding) {
    	ServletHost servletHost = null;
    	
        if (!hosts.isEmpty()) {
            servletHost = hosts.get(0);
        }
    	
        return new Axis2ServiceBindingProvider(component, service, binding,
                                               servletHost, modelFactories,
                                               policyHandlerClassnames, dataBindings, workScheduler);
    }
    
    public Class<WebServiceBinding> getModelType() {
        return WebServiceBinding.class;
    }
}
