/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.    
 */
package org.apache.tuscany.sca.implementation.java.impl;

import java.util.ArrayList;
import java.util.List;

import org.apache.tuscany.sca.implementation.java.IntrospectionException;
import org.apache.tuscany.sca.implementation.java.JavaImplementation;
import org.apache.tuscany.sca.implementation.java.JavaImplementationFactory;
import org.apache.tuscany.sca.implementation.java.introspect.JavaClassVisitor;

/**
 * A factory for the Java model.
 *
 * @version $Rev: 781195 $ $Date: 2009-06-02 23:37:24 +0100 (Tue, 02 Jun 2009) $
 */
public abstract class JavaImplementationFactoryImpl implements JavaImplementationFactory {
    
    private List<JavaClassVisitor> visitors = new ArrayList<JavaClassVisitor>();
    private JavaClassIntrospectorImpl introspector;
    
    public JavaImplementationFactoryImpl() {
        introspector = new JavaClassIntrospectorImpl(visitors);
    }

    public JavaImplementation createJavaImplementation() {
        JavaImplementation javaImplementation = new JavaImplementationImpl();
        return javaImplementation;
    }
    
    public JavaImplementation createJavaImplementation(Class<?> implementationClass) throws IntrospectionException {
        JavaImplementation javaImplementation = createJavaImplementation();
        introspector.introspectClass(javaImplementation, implementationClass);
        return javaImplementation;
    }
    
    public void createJavaImplementation(JavaImplementation javaImplementation, Class<?> implementationClass) throws IntrospectionException {
        introspector.introspectClass(javaImplementation, implementationClass);
    }

    public void addClassVisitor(JavaClassVisitor visitor) {
    	for (JavaClassVisitor tmpVisitor : visitors){
    		if (tmpVisitor.getClass() == visitor.getClass()){
    			// trying to add a duplicate visitor so
    			// ignore it 
    			return;
    		}
    	}
        visitors.add(visitor);
    }

    public void removeClassVisitor(JavaClassVisitor visitor) {
        visitors.remove(visitor);
    }
    
    public List<JavaClassVisitor> getClassVisitors() {
        return visitors;
    }

}
